(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('js-dynamic-function')) :
  typeof define === 'function' && define.amd ? define(['exports', 'js-dynamic-function'], factory) :
  (factory((global.JsBridgeAdapter = {}),global.DynamicFunction));
}(this, (function (exports,DynamicFunction) { 'use strict';

  var DynamicFunction__default = 'default' in DynamicFunction ? DynamicFunction['default'] : DynamicFunction;

  // 值类型判断 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  var isUndefined = function isUndefined(val) {
    return typeof val === 'undefined';
  };

  var isFunction = function isFunction(val) {
    return typeof val === 'function';
  };

  var isString = function isString(val) {
    return typeof val === 'string';
  };
  // 值类型判断 -------------------------------------------------------------

  var get = function get(obj) {
    var keys = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
    var defaultValue = arguments[2];

    try {
      var result = (isString(keys) ? keys.split('.') : keys).reduce(function (res, key) {
        return res[key];
      }, obj);
      return isUndefined(result) ? defaultValue : result;
    } catch (e) {
      return defaultValue;
    }
  };

  var run = function run(obj) {
    for (var _len = arguments.length, args = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
      args[_key - 2] = arguments[_key];
    }

    var keys = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

    keys = isString(keys) ? keys.split('.') : keys;

    var func = get(obj, keys);
    var context = get(obj, keys.slice(0, -1));

    return isFunction(func) ? func.call.apply(func, [context].concat(args)) : func;
  };

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  var parseConfig = function parseConfig(config) {
    if (typeof config === 'function') {
      return {
        runner: config
      };
    }

    return config;
  };

  var Api = function Api() {
    var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _classCallCheck(this, Api);

    var _parseConfig = parseConfig(config),
        _parseConfig$runner = _parseConfig.runner,
        runner = _parseConfig$runner === undefined ? Api.default.runner : _parseConfig$runner,
        _parseConfig$name = _parseConfig.name,
        name = _parseConfig$name === undefined ? get(runner, 'name') : _parseConfig$name,
        _parseConfig$isSuppor = _parseConfig.isSupported,
        isSupported = _parseConfig$isSuppor === undefined ? Api.default.isSupported : _parseConfig$isSuppor,
        _parseConfig$getRunne = _parseConfig.getRunner,
        getExecutor = _parseConfig$getRunne === undefined ? DynamicFunction.gen(isSupported, function (isSupported) {
      return isSupported && isFunction(runner) ? runner : undefined;
    }) : _parseConfig$getRunne;

    var func = new DynamicFunction__default({ name: name, getExecutor: getExecutor });

    func.isSupported = func.isExecutable;
    func.getRunner = func.getExecutor;

    delete func.isExecutable;
    delete func.getExecutor;

    return func;
  };

  Api.default = {
    isSupported: function isSupported() {
      return true;
    },
    runner: undefined
  };

  function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

  function _classCallCheck$1(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  var Bridge = function Bridge() {
    var _this = this;

    var name = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'anonymous';

    _classCallCheck$1(this, Bridge);

    this.apis = {};
    this.silent = Bridge.default.silent;

    this.bridgeSupport = function () {
      return false;
    };

    this.getRunner = function () {
      return function () {
        return undefined;
      };
    };

    this.log = function (level) {
      for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        args[_key - 1] = arguments[_key];
      }

      if (_this.silent) {
        return;
      }

      run.apply(undefined, [console, level].concat(args));
    };

    this.config = function () {
      var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      var support = _ref.support,
          api = _ref.api,
          rest = _objectWithoutProperties(_ref, ['support', 'api']);

      return Object.assign(_this, rest, {
        getRunner: api,
        bridgeSupport: support
      });
    };

    this.register = function () {
      var apis = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      Object.assign(_this.apis, run(apis, undefined, _this.api));
      return _this;
    };

    this.api = function (key) {
      var _ref2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
          _ref2$getRunner = _ref2.getRunner,
          _getRunner = _ref2$getRunner === undefined ? function () {
        return _this.getRunner(key);
      } : _ref2$getRunner;

      var bridgeRunner = new Api({
        name: key,
        runner: function runner() {
          for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
            args[_key2] = arguments[_key2];
          }

          return run.apply(undefined, [_getRunner(), undefined].concat(args));
        },
        isSupported: function isSupported() {
          return _this.bridgeSupport(key);
        }
      });

      bridgeRunner.customize = function (getCustomizedRunner) {
        return _this.api(key, {
          getRunner: function getRunner() {
            return getCustomizedRunner(_getRunner());
          }
        });
      };

      return bridgeRunner;
    };

    this.support = function (key) {
      return run(_this.apis, key + '.isSupported');
    };

    this.has = function (key) {
      return key in _this.apis;
    };

    this.get = function (key) {
      return get(_this.apis, key);
    };

    this.call = function (key) {
      for (var _len3 = arguments.length, args = Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
        args[_key3 - 1] = arguments[_key3];
      }

      var unregisteredTips = '[Unregistered] Api "' + key + '" is unregistered in Bridge "' + _this.name + '"';
      var notSupportedTips = '[Not Supported] Api "' + key + '" is not supported by Bridge "' + _this.name;

      if (!_this.has(key)) {
        return _this.log('warn', unregisteredTips);
      }

      return DynamicFunction.gen(function () {
        return run(_this.apis, key + '.getRunner');
      }, function (runner) {
        if (!isFunction(runner)) {
          return _this.log('warn', notSupportedTips);
        }

        return runner.apply(undefined, args);
      })();
    };

    this.name = name;
  };

  Bridge.Api = Api;
  Bridge.default = {
    silent: false
  };

  var uuid = 0;

  var uniqueId = function uniqueId() {
    var prefix = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
    return prefix + '_' + ++uuid + '_' + Date.now() + '_' + Math.floor(Math.random() * 1000000);
  };

  var getGlobal = function getGlobal() {
    if (typeof self !== 'undefined') {
      return self;
    }
    if (typeof window !== 'undefined') {
      return window;
    }
    if (typeof global !== 'undefined') {
      return global;
    }
    throw new Error('unable to locate global object');
  };

  var globalThis = getGlobal();

  var globalize = function globalize(handler) {
    var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
        _ref$name = _ref.name,
        name = _ref$name === undefined ? uniqueId('__globalFunction') : _ref$name,
        _ref$once = _ref.once,
        once = _ref$once === undefined ? false : _ref$once;

    if (isString(handler)) {
      return handler;
    }

    globalThis[name] = function () {
      if (once) {
        delete globalThis[name];
      }

      if (isFunction(handler)) {
        return handler.apply(undefined, arguments);
      }

      return;
    };

    return name;
  };

  exports.default = Bridge;
  exports.Api = Api;
  exports.Bridge = Bridge;
  exports.globalize = globalize;
  exports.uniqueId = uniqueId;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
